/** Copyright (C) 2017 by Bundesanstalt für Gewässerkunde
 * Software engineering by
 *  Björnsen Beratende Ingenieure GmbH
 *  Dr. Schumacher Ingenieurbüro für Wasser und Umwelt
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */
package org.dive4elements.river.client.client.ui;

import org.dive4elements.river.client.shared.model.Artifact;
import org.dive4elements.river.client.shared.model.User;

/**
 * Holds all data that configures the {@link DatacageWidget}
 *
 * @author Gernot Belger
 */
public class DatacageWidgetData {

    private final Artifact artifact;
    private final User user;
    private final String outs;
    private final String parameters;
    private final boolean showButton;
    private final String columnLabel;

    public DatacageWidgetData(final Artifact artifact, final User user) {
        this(artifact, user, null);
    }

    public DatacageWidgetData(final Artifact artifact, final User user, final String outs) {
        this(artifact, user, outs, true);
    }

    public DatacageWidgetData(final Artifact artifact, final User user, final String outs, final boolean showButton) {
        this(artifact, user, outs, null, showButton);
    }

    public DatacageWidgetData(final Artifact artifact, final User user, final String outs, final String parameters, final boolean showButton) {
        this(artifact, user, outs, parameters, showButton, null);
    }

    public DatacageWidgetData(final Artifact artifact, final User user, final String outs, final String parameters, final boolean showButton,
            final String columnLabel) {
        this.artifact = artifact;
        this.user = user;
        this.outs = outs;
        this.parameters = parameters;
        this.showButton = showButton;
        this.columnLabel = columnLabel;
    }

    public Artifact getArtifact() {
        return this.artifact;
    }

    public User getUser() {
        return this.user;
    }

    public String getOuts() {
        return this.outs;
    }

    public String getParameters() {
        return this.parameters;
    }

    public boolean isShowButton() {
        return this.showButton;
    }

    public String getColumnLabel() {
        return this.columnLabel;
    }
}